"""Tests for taxa module - search methods"""
import requests

from pyobis import taxa


def test_taxa_search_data():
    """
    taxa.search - basic test for data, check type, size and other methods
    """
    query = taxa.search(scientificname="Mola mola")
    assert not query.data  # the data is none after query building but before executing
    query.execute()
    assert "dict" == query.data.__class__.__name__
    assert list == list(query.data.keys()).__class__
    assert 2 == len(query.data)


def test_taxa_search_url():
    """
    taxa.search - basic test for url, url are accessible and
    mapper_url is None for unsupported methods
    """
    query = taxa.search(scientificname="Mola mola")
    assert requests.get(query.api_url).status_code == 200
    assert not query.mapper_url


def test_taxa_taxon_data():
    """
    taxa.taxon - basic test for data, check type, size and other methods
    """
    query = taxa.taxon(545439)
    assert not query.data
    query.execute()
    assert dict == query.data.__class__
    assert 2 == len(query.data)
    assert list == list(query.data.keys()).__class__
    assert 545439 == query.data["results"][0]["taxonID"]
    assert query.to_pandas().__class__.__name__ == "DataFrame"


def test_taxa_taxon_url():
    """
    taxa.taxon - basic test for url, url are accessible and
    mapper_url is None for unsupported methods
    """
    query = taxa.taxon(545439)
    assert requests.get(query.api_url).status_code == 200
    assert not query.mapper_url


def test_taxa_annotations_data():
    """
    taxa.annotations - basic test for data, check type, size and other methods
    """
    query = taxa.annotations(scientificname="Abra")
    assert not query.data
    query.execute()
    assert dict == query.data.__class__
    assert 2 == len(query.data)
    assert list == list(query.data.keys()).__class__
    assert query.to_pandas().__class__.__name__ == "DataFrame"


def test_taxa_annotations_url():
    """
    taxa.annotations - basic test for url, url are accessible and
    mapper_url is None for unsupported methods
    """
    query = taxa.annotations(scientificname="Abra")
    assert requests.get(query.api_url).status_code == 200
    assert not query.mapper_url



"""
pyobis library
~~~~~~~~~~~~~~~~~~~~~

pyobis is a Python client for OBIS.

Example usage:

# Import entire library
import pyobis
# or import modules as needed
## occurrence
from pyobis import occurrence
## taxa
from pyobis import taxa
## dataset
from pyobis import dataset
## checklist
from pyobis import checklist

## use advanced logging
### setup first
import requests
import logging
import httplib as http_client
http_client.HTTPConnection.debuglevel = 1
logging.basicConfig()
logging.getLogger().setLevel(logging.DEBUG)
requests_log = logging.getLogger("requests.packages.urllib3")
requests_log.setLevel(logging.DEBUG)
requests_log.propagate = True
### then make request
from pyobis import occurrence
occurrence.search(geometry='POLYGON((30.1 10.1, 10 20, 20 40, 40 40, 30.1 10.1))')
"""

try:
    from ._version import __version__
except ImportError:
    __version__ = "unknown"

__title__ = "pyobis"
__author__ = "pyOBIS Community"
__license__ = "MIT"

from .checklist import checklist
from .dataset import dataset
from .nodes import nodes
from .occurrences import occurrences
from .taxa import taxa

__all__ = [
    "checklist",
    "dataset",
    "nodes",
    "occurrences",
    "taxa",
]



"""
/taxon/ API endpoints as documented on https://api.obis.org/.
"""
import pandas as pd

from ..obisutils import build_api_url, handle_arrstr, obis_baseurl, obis_GET


def search(scientificname=None, **kwargs):
    """
    Get taxon records.

    :param scientificname: [String,Array] One or more scientific names from the
        OBIS backbone. All included and synonym taxa are included in the search

    :return: A dictionary

    Usage::

        from pyobis import taxa

        # build a query
        query = taxa.search(scientificname = 'Mola mola')
        query.execute() # execute the query i.e. fetch the data
        query.data # return the fetched data
        query.api_url # get the OBIS API URL for the built query
        query.mapper_url # get the OBIS Mapper URL (if it exists)

        # or simply get the data in one-easy step
        data = taxa.search(scientificname=['Mola mola','Abra alba']).execute()
    """

    scientificname = handle_arrstr(scientificname)
    url = obis_baseurl + "taxon/" + scientificname
    args = {"scientificname": scientificname}
    # return a taxa response class

    return TaxaResponse(url, args)


def taxon(id, **kwargs):
    """
    Get taxon by ID

    :param id: [Fixnum] An OBIS taxon identifier

    :return: A TaxaResponse object

    Usage::

        from pyobis import taxa
        query1 = taxa.taxon(545439)
        query1.execute()
        # executes the query previously built from OBIS
        query1.data
        # returns the data
        query1.api_url
        # returns the OBIS API URL
        query1.mapper_url
        # returns the OBIS Mapper URL for easy visualization

        taxa.taxon(402913).execute()
        q2 = taxa.taxon(406296)
        q3 = taxa.taxon(415282)
    """
    url = obis_baseurl + "taxon/" + str(id)
    args = {}
    # return a TaxaResponse Object
    return TaxaResponse(url, args)


def annotations(scientificname, **kwargs):
    """
    Get scientific name annotations by the WoRMS team.

    :param scientificname: [String] Scientific name. Leave empty to include all taxa.
    :return: A TaxaResponse Object

    Usage::

        from pyobis import taxa
        query1 = taxa.annotations(scientificname="Abra")
        query1.execute()
        # executes the query previously built from OBIS
        query1.data
        # returns the data
        query1.api_url
        # returns the OBIS API URL
        query1.mapper_url
        # returns the OBIS Mapper URL for easy visualization

    """
    url = obis_baseurl + "taxon/annotations"
    scientificname = handle_arrstr(scientificname)
    args = {"scientificname": scientificname}
    # return a TaxaResponse Object
    return TaxaResponse(url, args)


class TaxaResponse:
    """
    An OBIS Taxa Response Class
    """

    def __init__(self, url, args):
        """
        Initialise the object parameters
        """
        # public members
        self.data = None
        self.api_url = build_api_url(url, args)
        self.mapper_url = None

        # private members
        self.__args = args
        self.__url = url

    def execute(self, **kwargs):
        """
        Execute or fetch the data based on the query
        """
        out = obis_GET(
            self.__url, self.__args, "application/json; charset=utf-8", **kwargs
        )
        self.data = out
        return self.data

    def to_pandas(self):
        """
        Convert the results into a pandas DataFrame
        """
        return pd.DataFrame(self.data["results"])



"""
/dataset/ API endpoints as documented on https://api.obis.org/.
"""

import pandas as pd

from ..obisutils import build_api_url, handle_arrstr, obis_baseurl, obis_GET


def search(
    scientificname=None,
    taxonid=None,
    nodeid=None,
    startdate=None,
    enddate=None,
    startdepth=None,
    enddepth=None,
    geometry=None,
    flags=None,
    limit=None,
    offset=0,
    **kwargs,
):
    """
    Find dataset records.

    :param taxonid: [Fixnum] A obis Taxon AphiaID.
    :param scientificname: [String, Array] One or more scientific names from
        the OBIS backbone. All included and synonym taxa are
        included in the search.
    :param year: [Fixnum] The 4 digit year. A year of 98 will be interpreted
        as AD 98. Supports range queries,
        smaller,larger (e.g., '1990,1991', whereas '1991,1990' wouldn't work)
    :param geometry: [String] Well Known Text (WKT). A WKT shape written as
        either POINT, LINESTRING, LINEARRING
        or POLYGON.
        Example of a polygon: ((30.1 10.1, 20, 20 40, 40 40, 30.1 10.1)) would
        be queried as http://bit.ly/1BzNwDq. Geometry, formatted as WKT or
        GeoHash.
    :param nodeid: [Fixnum] Node UUID.
    :param startdate: [Fixnum] Start date
    :param enddate: [Boolean] End date
    :param startdepth: [Fixnum] Start depth, in meters. Depth below sea level are treated as
        positive numbers.
    :param enddepth: [Fixnum] End depth, in meters. Depth below sea level are treated as
        positive numbers.
    :param flags: [String, Array] Comma separated list of quality flags that
        need to be set
    :param offset: [Fixnum] Start at record. Default: 0

    :return: A DatasetResponse object

    Usage::

        from pyobis import dataset
        query = dataset.search(scientificname = 'Mola mola')
        data = query.execute() # or query.data

        # Many names
        dataset.search(
            scientificname = ['Mola', 'Abra', 'Lanice', 'Pectinaria']
        ).execute()

        # Use paging parameters (limit and start) to page.
        # Note the different results for the two queries below.
        # build the query
        query = dataset.search(scientificname = 'Mola mola')
        query.execute() # fetch the data
        query.data # return the data
        dataset.search(scientificname = 'Mola mola').execute() # or simply one-easy step

        # Search on a bounding box
        ## in well known text format
        dataset.search(
            geometry='POLYGON((30.1 10.1, 10 20, 20 40, 40 40, 30.1 10.1))'
        ).execute()
        from pyobis import taxa
        from pyobis import dataset

        res = taxa.search(scientificname='Mola mola').execute().data['results'][0]
        dataset.search(
            taxonid=res['id'],
            geometry='POLYGON((30.1 10.1, 10 20, 20 40, 40 40, 30.1 10.1))',
            limit=20
        ).execute()
        dataset.search(
            aphiaid=res['worms_id'],
            geometry='POLYGON((30.1 10.1, 10 20, 20 40, 40 40, 30.1 10.1))',
            limit=20
        ).execute()

        # Get resources for a particular eventDate
        data = dataset.search(taxonid=res['worms_id']).execute()
    """
    url = obis_baseurl + "dataset"
    scientificname = handle_arrstr(scientificname)
    args = {
        "taxonid": taxonid,
        "nodeid": nodeid,
        "scientificname": scientificname,
        "startdate": startdate,
        "enddate": enddate,
        "startdepth": startdepth,
        "enddepth": enddepth,
        "geometry": geometry,
        "offset": offset,
        "flags": flags,
        "size": limit,
    }

    mapper = False
    return DatasetResponse(url, args, mapper)


def get(id, **kwargs):
    """
    Get dataset by ID

    :param id: [Fixnum] An OBIS dataset identifier.

    :return: A DatasetResponse object

    Usage::

        from pyobis import dataset
        query = dataset.get('ec9df3b9-3b2b-4d83-881b-27bcbcd57b95')
        query.execute() # execute the query
        query.data # returns the data of the query
        query.api_url # returns the OBIS API URL
    """
    url = obis_baseurl + "dataset/" + str(id)
    args = {}
    mapper = True

    # returns a DatasetResponse object
    return DatasetResponse(url, args, mapper)


class DatasetResponse:
    """
    An OBIS Dataset Response Object
    """

    def __init__(self, url, args, mapper):
        """
        Initialise the object parameters
        """
        # public members
        self.data = None
        self.api_url = build_api_url(url, args)
        self.mapper_url = None
        if mapper:
            self.mapper_url = f"https://mapper.obis.org/?datasetid={url.split('/')[-1]}"

        # private members
        self.__args = args
        self.__url = url

    def execute(self, **kwargs):
        """
        Execute or fetch the data based on the query
        """
        out = obis_GET(
            self.__url, self.__args, "application/json; charset=utf-8", **kwargs
        )
        self.data = out
        return self.data

    def to_pandas(self):
        """
        Convert the results into a pandas DataFrame
        """
        return pd.DataFrame(self.data["results"])



from .occurrences import (
    OccResponse,
    centroid,
    get,
    getpoints,
    grid,
    lookup_taxon,
    point,
    search,
    tile,
)

__all__ = [
    "search",
    "get",
    "getpoints",
    "grid",
    "tile",
    "lookup_taxon",
    "point",
    "centroid",
    "OccResponse",
]



"""Tests for nodes module"""
import requests

from pyobis import nodes


def test_nodes_search_data():
    """
    nodes.search - test for data, check type, size and other methods
    """
    query = nodes.search(id="4bf79a01-65a9-4db6-b37b-18434f26ddfc")
    assert not query.data
    query.execute()
    assert "dict" == query.data.__class__.__name__
    assert 2 == len(query.data)
    assert dict == query.data["results"][0].__class__
    assert str == str(query.data["results"][0]["description"]).__class__
    assert str == query.data["results"][0]["id"].__class__
    assert query.to_pandas().__class__.__name__ == "DataFrame"


def test_nodes_search_url():
    """
    nodes.activities - basic test for url, url are accessible and
    mapper_url correct for supported methods
    """
    query = nodes.search(id="4bf79a01-65a9-4db6-b37b-18434f26ddfc")
    assert requests.get(query.api_url).status_code == 200
    assert requests.get(query.mapper_url).status_code == 200


def test_nodes_activities_data():
    """
    nodes.activities - basic test for data, check type, size and other methods
    """
    query = nodes.activities(id="4bf79a01-65a9-4db6-b37b-18434f26ddfc")
    assert not query.data
    query.execute()
    assert "dict" == query.data.__class__.__name__
    assert 2 == len(query.data)
    assert dict == query.data["results"][0].__class__
    assert str == str(query.data["results"][0]["description"]).__class__
    assert str == query.data["results"][0]["id"].__class__
    assert query.to_pandas().__class__.__name__ == "DataFrame"


def test_nodes_activities_url():
    """
    nodes.activities - basic test for url, url are accessible and
    mapper_url correct for supported methods
    """
    query = nodes.activities(id="4bf79a01-65a9-4db6-b37b-18434f26ddfc")
    assert requests.get(query.api_url).status_code == 200
    assert not query.mapper_url



"""
Utility functions for internal use across various modules.
"""
from urllib.parse import urlencode

import requests

obis_baseurl = "https://api.obis.org/v3/"


class NoResultException(Exception):
    """
    Thrown when query returns no results.
    """

    pass


def build_api_url(url, args):
    """
    Builds the API URL based on the base url and the arguments
    """
    return url + "?" + urlencode({k: v for k, v in args.items() if v is not None})


def obis_GET(url, args, ctype, **kwargs):
    """
    Handles technical details of sending GET request to the API
    """
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko)\
         Chrome/107.0.0.0 Safari/537.36 Edg/107.0.1418.52",
        "Accept-Encoding": "gzip, deflate, br",
        "Host": "api.obis.org",
        "Connection": "keep-alive",
    }
    out = requests.get(url, params=args, headers=headers, **kwargs)
    out.raise_for_status()
    stopifnot(out.headers["content-type"], ctype)
    return out.json()


def obis_write_disk(url, path, ctype, **kwargs):
    out = requests.get(url, stream=True, **kwargs)
    out.raise_for_status()
    with open(path, "wb") as f:
        for chunk in out.iter_content(chunk_size=1024):
            if chunk:
                f.write(chunk)
    return path


def stopifnot(x, ctype):
    if x != ctype:
        raise NoResultException("content-type did not equal " + str(ctype))


def stop(x):
    raise ValueError(x)


def handle_arrstr(x):
    """Converts array arguments into comma-separated strings if applicable."""
    if x.__class__.__name__ == "NoneType":
        pass
    else:
        if x.__class__.__name__ == "str":
            return x
        else:
            return ",".join(x)


def handle_arrint(x):
    """Converts array arguments into comma-separated integers if applicable."""
    if x.__class__.__name__ == "NoneType":
        pass
    else:
        if x.__class__.__name__ == "int":
            return x
        else:
            x = list(map(str, x))
            return ",".join(x)



"""Tests for occurrences module methods"""
import requests

from pyobis import occurrences


def test_occurrences_search():
    """
    occurrences.search - basic test for data, check type, size and other methods
    """
    size = 10100
    query = occurrences.search(scientificname="Mola mola", size=size)
    assert not query.data  # the data is none after query building but before executing
    query.execute()
    assert size == len(query.data)
    assert "Mola mola" == query.data.scientificName[0]


def test_occurrence_search_mof():
    """
    occurrences.search - basic test for data with MoF extension, check type, size and other methods
    """
    query = occurrences.search(
        scientificname="Abra alba",
        mof=True,
        size=100,
        hasextensions="MeasurementOrFact",
    )
    assert not query.data
    query.execute()
    assert "Abra alba" == query.data.scientificName[0]
    assert requests.get(query.api_url).status_code == 200
    assert requests.get(query.mapper_url).status_code == 200


def test_occurrences_search_61():
    """
    Search returns same object-type regardless of mof=True or mof=False.
    Tests for https://github.com/iobis/pyobis/issues/61.
    """
    TEST_QUERY = dict(
        scientificname="Mola mola",
        size=2,
    )
    q1 = occurrences.search(mof=True, **TEST_QUERY).execute()
    q2 = occurrences.search(mof=False, **TEST_QUERY).execute()

    assert type(q1) == type(q2)


def test_occurrences_get():
    """
    occurrences.get - basic test for data, check type, size and other methods
    """
    query = occurrences.get(id="00003cf7-f2fc-4c53-98a6-7d846e70f5d1")
    assert not query.data
    query.execute()
    assert "dict" == query.data.__class__.__name__
    assert 2 == len(query.data)
    assert list == list(query.data.keys()).__class__
    assert requests.get(query.api_url).status_code == 200
    assert query.to_pandas().__class__.__name__ == "DataFrame"


def test_occurrences_grid():
    """
    occurrences.grid - basic test for data, check type, size and other methods
    """
    query = occurrences.grid(5, geojson=True, scientificname="Abra alba")
    assert not query.data
    query.execute()
    assert "dict" == query.data.__class__.__name__
    assert 2 == len(query.data)
    assert list == list(query.data.keys()).__class__
    query = occurrences.grid(5, geojson=False, scientificname="Mola mola")
    assert requests.get(query.api_url).status_code == 200
    assert not query.mapper_url


def test_occurrences_getpoints():
    """
    occurrences.getpoints - basic test for data, check type, size and other methods
    """
    query = occurrences.getpoints(scientificname=["Mola mola", "Abra alba"])
    assert not query.data
    query.execute()
    assert "dict" == query.data.__class__.__name__
    assert 2 == len(query.data)
    assert list == list(query.data.keys()).__class__
    assert requests.get(query.api_url).status_code == 200
    assert not query.mapper_url


def test_occurrences_point():
    """
    occurrences.point - basic test for data, check type, size and other methods
    """
    query = occurrences.point(x=1.77, y=54.22, scientificname="Mola mola")
    assert not query.data
    query.execute()
    assert "dict" == query.data.__class__.__name__
    assert 2 == len(query.data)
    assert list == list(query.data.keys()).__class__
    assert requests.get(query.api_url).status_code == 200
    assert not query.mapper_url


def test_occurrences_tile():
    """
    occurrences.tile - basic test for data, check type, size and other methods
    """
    query = occurrences.tile(x=1.77, y=52.26, z=0.5, mvt=0, scientificname="Mola mola")
    assert not query.data
    query.execute()
    assert "dict" == query.data.__class__.__name__
    assert 2 == len(query.data)
    assert list == list(query.data.keys()).__class__
    query = occurrences.tile(x=1.77, y=52.26, z=0.5, mvt=1, scientificname="Mola mola")
    query.execute()
    assert requests.get(query.api_url).status_code == 200
    query = occurrences.tile(x=1.77, y=52.26, z=0.5, mvt=0, scientificname="Mola mola")
    query.execute()
    assert requests.get(query.api_url).status_code == 200
    assert not query.mapper_url


def test_occurrences_centroid():
    """
    occurrences.centroid - basic test for data, check type, size and other methods
    """
    query = occurrences.centroid(scientificname="Mola mola")
    assert not query.data
    query.execute()
    assert "dict" == query.data.__class__.__name__
    assert 2 == len(query.data)
    assert list == list(query.data.keys()).__class__
    assert requests.get(query.api_url).status_code == 200
    assert not query.mapper_url



"""
/nodes/ API endpoints as documented on https://api.obis.org/.
"""

import pandas as pd

from ..obisutils import build_api_url, obis_baseurl, obis_GET


def search(id=None, **kwargs):
    """
    Get OBIS nodes records

    :param id: [String] Node UUID.

    :return: A NodesQuery Object

    Usage::

        from pyobis import nodes
        data = nodes.search(id="4bf79a01-65a9-4db6-b37b-18434f26ddfc").execute()
    """
    url = obis_baseurl + "node/" + id
    mapper = True
    args = {}

    # return NodesQuery Object
    return NodesResponse(url, args, mapper)


def activities(id=None, **kwargs):
    """
    Get OBIS nodes activities

    :param id: [String] Node UUID.

    :return: A NodesQuery object

    Usage::

        from pyobis import nodes

        # build a query
        query = nodes.activities(id="4bf79a01-65a9-4db6-b37b-18434f26ddfc")
        query.execute() # execute the query to fetch the data
        query.data # get fetched data
        query.api_url # check the corresponding OBIS API URL for the query
        query.mapper_url # check the corresponding OBIS Mapper URL for the query (if it exists)
    """
    url = obis_baseurl + "node/" + id + "/activities"
    args = {}
    mapper = False

    # return a NodesQuery object
    return NodesResponse(url, args, mapper)


class NodesResponse:
    """
    An OBIS Nodes Response Class
    """

    def __init__(self, url, args, mapper):
        """
        Initialize NodesResponse Object
        """
        # public members
        self.data = None
        self.api_url = build_api_url(url, args)
        self.mapper_url = None
        if mapper:
            # get the node id from already built url
            self.mapper_url = f"https://mapper.obis.org/?nodeid={url.split('/')[-1]}"

        # private members
        self.__args = args
        self.__url = url

    def execute(self, **kwargs):
        """
        Fetch data for Nodes Query
        """
        out = obis_GET(
            self.__url, self.__args, "application/json; charset=utf-8", **kwargs
        )
        self.data = out
        return self.data

    def to_pandas(self):
        """
        Convert fetched data to a pandas DataFrame
        """
        if "contacts" in self.data["results"][0].keys():
            return pd.merge(
                pd.DataFrame(self.data["results"]),
                pd.json_normalize(self.data["results"], "contacts", "id"),
                on="id",
                how="inner",
            ).drop("contacts", axis=1)

        return pd.merge(
            pd.DataFrame(self.data["results"]),
            pd.json_normalize(self.data["results"], "contributions", "id"),
            on="id",
            how="inner",
        ).drop("contributions", axis=1)



"""
/checklist/ API endpoints as documented on https://api.obis.org/.
"""
import sys

import pandas as pd

from ..obisutils import (
    build_api_url,
    handle_arrint,
    handle_arrstr,
    obis_baseurl,
    obis_GET,
)


class ChecklistResponse:
    """
    An OBIS Checklist Response Object
    """

    def __init__(self, url, args, paginate):
        """
        Initialise the object parameters
        """
        self.api_url = build_api_url(url, args)
        self.mapper_url = None
        self.data = None

        # private members
        self.__url = url
        self.__args = args
        self.__paginate = paginate

    def execute(self):
        """
        Execute or fetch the data based on the query
        """
        if self.__paginate:
            out = obis_GET(
                self.__url,
                self.__args,
                "application/json; charset=utf-8",
            )
            self.__args["skip"] += 10
            self.__args["size"] = 5000
            i = 10

            # an error check is necessary, otherwise print statement throws "division by zero" error
            try:
                out["error"]
                return out["error"]
            except KeyError:
                pass

            # fetch first 10 records, and print number of estimated records
            print(f"Estimated records: {out['total']}")
            print(
                "{}[{}{}] {}".format(
                    "Fetching: ",
                    "█" * int(len(out["results"]) * 100 / out["total"]),
                    "." * (100 - int(len(out["results"]) * 100 / out["total"])),
                    len(out["results"]),
                ),
                end="\r",
                file=sys.stdout,
                flush=True,
            )
            # now paginate until the response is null
            while True:
                res = obis_GET(
                    self.__url,
                    self.__args,
                    "application/json; charset=utf-8",
                )
                # when we find that no records are there, we break out of loop
                if len(res["results"]) == 0:
                    break
                out["results"] += res["results"]
                # print the progress bar
                print(
                    "{}[{}{}] {}".format(
                        "Fetching: ",
                        "█" * int(len(out["results"]) * 100 / out["total"]),
                        "." * (100 - int(len(out["results"]) * 100 / out["total"])),
                        len(out["results"]),
                    ),
                    end="\r",
                    file=sys.stdout,
                    flush=True,
                )
                self.__args["skip"] = len(out["results"])
                # continue to fetch next 5000 records
                i += 5000
            # print actual number of fetched records
            print(f"\nFetched {len(out['results'])} records.")
        else:
            out = obis_GET(
                self.__url,
                self.__args,
                "application/json; charset=utf-8",
            )
        self.data = out

    def to_pandas(self):
        """
        Convert the results into a pandas DataFrame
        """
        return pd.DataFrame(self.data["results"])


def list(
    scientificname=None,
    taxonid=None,
    nodeid=None,
    startdate=None,
    enddate=None,
    startdepth=None,
    enddepth=None,
    geometry=None,
    flags=None,
    **kwargs,
):
    """
    Generate an OBIS checklist

    :param taxonid: [Fixnum] A obis occurrence identifier
    :param scientificname: [String,Array] One or more scientific names from
        the OBIS backbone. All included and synonym taxa are included in
        the search.
    :param geometry: [String] Well Known Text (WKT). A WKT shape written as
        either POINT, LINESTRING, LINEARRING
        or POLYGON.
        Example of a polygon: ((30.1 10.1, 20, 20 40, 40 40, 30.1 10.1)) would
        be queried as http://bit.ly/1BzNwDq
    :param nodeid: [Fixnum] Node UUID.
    :param startdate: [String] Start date YYYY-MM-DD
    :param enddate: [String] End date YYYY-MM-DD
    :param startdepth: [Fixnum] Start depth, in meters. Depth below sea level are treated as
        positive numbers.
    :param enddepth: [Fixnum] End depth, in meters. Depth below sea level are treated as
        positive numbers.
    :param flags: [String] Comma separated list of quality flags which need
        to be set

    :return: A dictionary

    Usage::

        from pyobis import checklist
        checklist.list(scientificname = 'Mola mola').execute()

        # taxonid of 3013
        checklist.list(taxonid = 3013).execute()
    """
    url = obis_baseurl + "checklist"
    scientificname = handle_arrstr(scientificname)
    taxonid = handle_arrint(taxonid)
    args = {
        "taxonid": taxonid,
        "nodeid": nodeid,
        "scientificname": scientificname,
        "startdate": startdate,
        "enddate": enddate,
        "startdepth": startdepth,
        "enddepth": enddepth,
        "geometry": geometry,
        "flags": flags,
        "skip": 0,
        "size": 10,
    }

    return ChecklistResponse(url, args, paginate=True)


def redlist(
    scientificname=None,
    taxonid=None,
    nodeid=None,
    startdate=None,
    enddate=None,
    startdepth=None,
    enddepth=None,
    geometry=None,
    flags=None,
    **kwargs,
):
    """
    Generate a checklist of IUCN Red List species.

    :param scientificname: [String] Scientific name. Leave empty to include
        all taxa.
    :param taxonid: [String] Taxon AphiaID.
    :param nodeid: [String] Node UUID.
    :param startdate: [String] Start date formatted as YYYY-MM-DD.
    :param enddate: [String] End date formatted as YYYY-MM-DD.
    :param startdepth: [Fixnum] Start depth, in meters. Depth below sea level are treated as
        positive numbers.
    :param enddepth: [Fixnum] End depth, in meters. Depth below sea level are treated as
        positive numbers.
    :param geometry: [String] Geometry, formatted as WKT or GeoHash.
    :param flags: [String] Comma separated list of quality flags which need
        to be set.

    :return: A dictionary

    Usage::

        from pyobis import checklist
        checklist.redlist(scientificname='Abra Alba').execute()
    """
    url = obis_baseurl + "checklist/redlist"
    scientificname = handle_arrstr(scientificname)
    taxonid = handle_arrint(taxonid)
    args = {
        "taxonid": taxonid,
        "nodeid": nodeid,
        "scientificname": scientificname,
        "startdate": startdate,
        "enddate": enddate,
        "startdepth": startdepth,
        "enddepth": enddepth,
        "geometry": geometry,
        "flags": flags,
    }

    return ChecklistResponse(url, args, paginate=False)


def newest(
    scientificname=None,
    taxonid=None,
    nodeid=None,
    startdate=None,
    enddate=None,
    startdepth=None,
    enddepth=None,
    geometry=None,
    flags=None,
    **kwargs,
):
    """
    Generate a checklist of most recently added species.

    :param scientificname: [String] Scientific name. Leave empty to include
        all taxa.
    :param taxonid: [String] Taxon AphiaID.
    :param nodeid: [String] Node UUID.
    :param startdate: [String] Start date formatted as YYYY-MM-DD.
    :param enddate: [String] End date formatted as YYYY-MM-DD.
    :param startdepth: [Fixnum] Start depth, in meters. Depth below sea level are treated as
        positive numbers.
    :param enddepth: [Fixnum] End depth, in meters. Depth below sea level are treated as
        positive numbers.
    :param geometry: [String] Geometry, formatted as WKT or GeoHash.
    :param flags: [String] Comma separated list of quality flags which need to
        be set.

    :return: A dictionary

    Usage::

        from pyobis import checklist
        checklist.newest(scientificname='Abra Alba')
    """
    url = obis_baseurl + "checklist/newest"
    scientificname = handle_arrstr(scientificname)
    taxonid = handle_arrint(taxonid)
    args = {
        "taxonid": taxonid,
        "nodeid": nodeid,
        "scientificname": scientificname,
        "startdate": startdate,
        "enddate": enddate,
        "startdepth": startdepth,
        "enddepth": enddepth,
        "geometry": geometry,
        "flags": flags,
    }

    return ChecklistResponse(url, args, paginate=False)



"""
/occurrences/ API endpoints as documented on https://api.obis.org/.
"""

import json
import sys
from urllib.parse import urlencode

import pandas as pd
import requests

from ..obisutils import (
    build_api_url,
    handle_arrint,
    handle_arrstr,
    obis_baseurl,
    obis_GET,
)


class OccResponse:
    """
    An OBIS Occurrence response class
    """

    def __init__(self, url, args, isSearch, hasMapper, isKML):
        """ "
        Initialise the object parameters
        """
        self.data = None
        self.api_url = build_api_url(url, args)
        self.mapper_url = None
        if hasMapper:
            if not args["taxonid"] and args["scientificname"]:
                args["taxonid"] = lookup_taxon(args["scientificname"])[0]["id"]

            self.mapper_url = (
                "https://mapper.obis.org/"
                + "?"
                + urlencode(
                    {k: v for k, v in args.items() if v is not None},
                )
            )

        # private members
        self.__args = args
        self.__url = url
        self.__isSearch = isSearch
        self.__isKML = isKML

    def execute(self, **kwargs):
        """
        Execute or fetch the data based on the query
        """
        if not self.__isSearch and not self.__isKML:
            out = obis_GET(
                self.__url, self.__args, "application/json; charset=utf-8", **kwargs
            )

        elif self.__isKML:
            print("hello")
            out = requests.get(self.__url, params=self.__args, **kwargs)
            out.raise_for_status()
            out = out.content

        elif self.__isSearch:
            # setting default parameters from arguments list
            mof = self.__args["mof"]
            size = self.__args["size"]

            self.__args["size"] = 1
            out = obis_GET(
                self.__url, self.__args, "application/json; charset=utf-8", **kwargs
            )
            size = (
                out["total"] if not size else size
            )  # if the user has set some size or else we fetch all the records

            outdf = pd.DataFrame(columns=pd.DataFrame(out["results"]).columns)

            for i in range(5000, size + 1, 5000):
                self.__args["size"] = 5000
                print(
                    "{}[{}{}] {}/{}".format(
                        "Fetching: ",
                        "█" * int((i - 1) * 100 / size),
                        "." * (100 - int((i + 1) * 100 / size)),
                        i,
                        size,
                    ),
                    end="\r",
                    file=sys.stdout,
                    flush=True,
                )
                res = obis_GET(
                    self.__url, self.__args, "application/json; charset=utf-8", **kwargs
                )
                outdf = pd.concat(
                    [
                        outdf.infer_objects(),
                        pd.DataFrame(res["results"]).infer_objects(),
                    ],
                    ignore_index=True,
                )
                # make sure that we set the `after` parameter when fetching subsequent records
                self.__args["after"] = outdf["id"].iloc[-1]

            self.__args["size"] = size % 5000
            # we have already fetched records as a set of 5000 records each time,
            # now we need to get remaining records from the total
            print(
                "{}[{}{}] {}/{}".format("Fetching: ", "█" * 100, "." * 0, size, size),
                end="\r",
                file=sys.stdout,
                flush=True,
            )
            res = obis_GET(
                self.__url, self.__args, "application/json; charset=utf-8", **kwargs
            )
            outdf = pd.concat(
                [outdf.infer_objects(), pd.DataFrame(res["results"]).infer_objects()],
                ignore_index=True,
            )
            print(f"\nFetched {size} records.")

            if mof and out["total"] > 0:
                mofNormalized = pd.json_normalize(
                    json.loads(outdf.to_json(orient="records")),
                    "mof",
                    ["id"],
                )
                merged = pd.merge(
                    outdf,
                    mofNormalized,
                    on="id",
                    how="inner",
                )
                self.data = merged
                return self.data
            self.data = outdf
            return self.data
        self.data = out
        return self.data

    def to_pandas(self):
        """
        Convert the results into a pandas DataFrame
        """
        return pd.DataFrame(self.data["results"])


def get(id, **kwargs):
    """
    Get an OBIS occurrence

    :param id: [Fixnum] An obis occurrence identifier.
        It is returned in the 'id' field with occurrences.search().

    :return: A dictionary

    Usage::

        from pyobis import occurrences
        q1 = occurrences.get(id = '00003cf7-f2fc-4c53-98a6-7d846e70f5d1')
        q1.execute()
        q1.data # get the data
        q1.api_url # get the API url

    """
    url = obis_baseurl + "occurrence/" + str(id)
    args = {}

    return OccResponse(url, args, isSearch=False, hasMapper=False, isKML=False)


def search(
    scientificname=None,
    taxonid=None,
    nodeid=None,
    datasetid=None,
    startdate=None,
    enddate=None,
    startdepth=None,
    enddepth=None,
    geometry=None,
    year=None,
    flags=None,
    fields=None,
    size=None,
    offset=0,
    mof=False,
    hasextensions=None,
    **kwargs,
):
    """
    Search OBIS occurrences

    :param taxonid: [Fixnum] An OBIS occurrence identifier
    :param scientificname: [String,Array] One or more scientific names from the
        OBIS backbone. All included and
        synonym taxa are included in the search.
    :param year: Removed in v3 API. [Fixnum] The 4 digit year. A year of 98
        will be interpreted as AD 98. Supports range queries,
        smaller,larger (e.g., '1990,1991', whereas '1991,1990' wouldn't work)
    :param geometry: [String] Well Known Text (WKT). A WKT shape written
        as either POINT, LINESTRING, LINEARRING
        or POLYGON.
        Example of a polygon: ((30.1 10.1, 20, 20 40, 40 40, 30.1 10.1)) would
        be queried as http://bit.ly/1BzNwDq
    :param nodeid: [String] Node UUID
    :param taxonid: Prev. aphiaid [Fixnum] An Aphia id. This is listed as
        the `worms_id` in `taxa`/`taxon` results
    :param datasetid: Prev. resourceid [Fixnum] A resource id
    :param startdate: [Fixnum] Start date, formatted as YYYY-MM-DD
    :param enddate: [Boolean] End date, formatted as YYYY-MM-DD
    :param startdepth: [Fixnum] Start depth, in meters. Depth below sea level are treated
        as positive numbers.
    :param enddepth: [Boolean] End depth, in meters. Depth below sea level are treated
        as positive numbers.
    :param flags: Prev. qc [String] Quality control flags
    :param fields: [String] Comma seperated list of field names
    :param size: [Fixnum] Number of results to return. Default: All records
    :param offset: [Fixnum] Start at record. Default: 0
    :param mof: [Boolean] Include MeasurementOrFact records, true/false.
        Default: 0
    :param hasextensions: [String] Extensions that need to be present
        (e.g. MeasurementOrFact, DNADerivedData).
    :return: A dictionary

    Usage::

        from pyobis import occurrences
        occurrences.search(scientificname = 'Mola mola').execute()

        # Many names
        occurrences.search(scientificname = ['Mola', 'Abra', 'Lanice', 'Pectinaria']).execute()

        # Use paging parameters (limit and start) to page.
        # Note the different results for the two queries below.
        occ.search(scientificname = 'Mola mola', offset=0, size=10)
        occ.search(scientificname = 'Mola mola', offset=10, size=10)

        # Search on a bounding box
        ## in well known text format
        occurrences.search(
            geometry='POLYGON((30.1 10.1, 10 20, 20 40, 40 40, 30.1 10.1))',
            size=20
        ).execute()

        from pyobis import taxa

        res = taxa.search(scientificname='Mola mola').execute()['results'][0]
        occurrences.search(
            obisid=res['id'],
            geometry='POLYGON((30.1 10.1, 10 20, 20 40, 40 40, 30.1 10.1))',
            size=20
        ).execute()
        occurrences.search(
            aphiaid=res['worms_id'],
            geometry='POLYGON((30.1 10.1, 10 20, 20 40, 40 40, 30.1 10.1))',
            size=20
        ).execute()

        # Get mof response as a pandas dataframe
        occurrences.search(
            scientificname="Abra", mof=True, hasextensions="MeasurementOrFact", size=100
            ).execute()
    """
    url = obis_baseurl + "occurrence"
    scientificname = handle_arrstr(scientificname)
    taxonid = handle_arrint(taxonid)
    args = {
        "taxonid": taxonid,
        "nodeid": nodeid,
        "datasetid": datasetid,
        "scientificname": scientificname,
        "startdate": startdate,
        "enddate": enddate,
        "startdepth": startdepth,
        "enddepth": enddepth,
        "geometry": geometry,
        "year": year,
        "fields": fields,
        "flags": flags,
        "offset": offset,
        "mof": mof,
        "size": size,
        "hasextensions": hasextensions,
    }
    return OccResponse(url, args, isSearch=True, hasMapper=True, isKML=False)


def grid(
    precision,
    geojson=True,
    scientificname=None,
    taxonid=None,
    datasetid=None,
    nodeid=None,
    startdate=None,
    enddate=None,
    startdepth=None,
    enddepth=None,
    geometry=None,
    redlist=None,
    hab=None,
    wrims=None,
    event=None,
    flags=None,
    exclude=None,
    **kwargs,
):
    """
    Fetch gridded occurrences as GeoJSON or KML.

    :param precision: [integer] Geohash precision.
    :param scientificname: [string] Scientific name.
        Leave empty to include all taxa.
    :param taxonid: [string] Taxon AphiaID.
    :param datasetid: [string] Dataset UUID.
    :param nodeid: [string] Node UUID.
    :param startdate: [string] Start date formatted as YYYY-MM-DD.
    :param enddate: [string] End date formatted as YYYY-MM-DD.
    :param startdepth: [integer] Start depth, in meters. Depth below sea level are treated
        as positive numbers.
    :param enddepth: [integer] End depth, in meters. Depth below sea level are treated
        as positive numbers.
    :param geometry: [string] Geometry, formatted as WKT or GeoHash.
    :param redlist: [boolean] Red List species only, True/False.
    :param hab: [boolean] HAB species only, true/false.
    :param wrims: [boolean] WRiMS species only, True/False.
    :param event: [string] Include pure event records (include) or get pure
        event records exclusively (true).
    :param flags: [string] Comma separated list of required quality flags.
    :param exclude: [string] Comma separated list of quality flags
        to be excluded.

    :return: A dictionary

    Usage::

        from pyobis import occurrences

        occurrences.grid(100, True) // returns in GeoJSON format
        ococcurrences.grid(1000, False)   // returns in KML format
    """
    url = obis_baseurl + "occurrence/grid/" + str(precision)
    scientificname = handle_arrstr(scientificname)
    taxonid = handle_arrint(taxonid)
    args = {
        "scientificname": scientificname,
        "taxonid": taxonid,
        "datasetid": datasetid,
        "nodeid": nodeid,
        "startdate": startdate,
        "enddate": enddate,
        "startdepth": startdepth,
        "enddepth": enddepth,
        "geometry": geometry,
        "redlist": redlist,
        "hab": hab,
        "wrims": wrims,
        "event": event,
        "flags": flags,
        "exclude": exclude,
    }
    if not geojson:
        url += "/kml"
        return OccResponse(url, args, isSearch=False, hasMapper=False, isKML=True)

    return OccResponse(url, args, isSearch=False, hasMapper=False, isKML=False)


def getpoints(
    scientificname=None,
    taxonid=None,
    datasetid=None,
    nodeid=None,
    startdate=None,
    enddate=None,
    startdepth=None,
    enddepth=None,
    geometry=None,
    redlist=None,
    hab=None,
    wrims=None,
    event=None,
    flags=None,
    exclude=None,
    **kwargs,
):
    """
    Fetch point occurrences as GeoJSON (aggregated to Geohash precision 8).

    :param scientificname: [string] Scientific name. Leave empty to include all
        taxa.
    :param taxonid: [string] Taxon AphiaID.
    :param datasetid: [string] Dataset UUID.
    :param nodeid: [string] Node UUID.
    :param startdate: [string] Start date formatted as YYYY-MM-DD.
    :param enddate: [string] End date formatted as YYYY-MM-DD.
    :param startdepth: [integer] Start depth, in meters. Depth below sea level are treated
        as positive numbers.
    :param enddepth: [integer] End depth, in meters. Depth below sea level are treated
        as positive numbers.
    :param geometry: [string] Geometry, formatted as WKT or GeoHash.
    :param redlist: [boolean] Red List species only, True/False.
    :param hab: [boolean] HAB species only, true/false.
    :param wrims: [boolean] WRiMS species only, True/False.
    :param event: [string] Include pure event records (include) or get pure0
        event records exclusively (true).
    :param flags: [string] Comma separated list of quality flags which need to
        be set.
    :param exclude: [string] Comma separated list of quality flags to be
        excluded.

    :return: A dictionary

    Usage::

        from pyobis import occurrences
        occurrences.getpoints(scientificname = 'Mola mola')

        ## Many names
        occurrences.getpoints(scientificname = ['Mola mola','Abra alba'])
    """
    url = obis_baseurl + "occurrence/points"
    scientificname = handle_arrstr(scientificname)
    taxonid = handle_arrint(taxonid)
    args = {
        "scientificname": scientificname,
        "taxonid": taxonid,
        "datasetid": datasetid,
        "nodeid": nodeid,
        "startdate": startdate,
        "enddate": enddate,
        "startdepth": startdepth,
        "enddepth": enddepth,
        "geometry": geometry,
        "redlist": redlist,
        "hab": hab,
        "wrims": wrims,
        "event": event,
        "flags": flags,
        "exclude": exclude,
    }

    return OccResponse(url, args, isSearch=False, hasMapper=False, isKML=False)


def point(
    x,
    y,
    z=None,
    scientificname=None,
    taxonid=None,
    datasetid=None,
    nodeid=None,
    startdate=None,
    enddate=None,
    startdepth=None,
    enddepth=None,
    geometry=None,
    redlist=None,
    hab=None,
    wrims=None,
    event=None,
    flags=None,
    exclude=None,
    **kwargs,
):
    """
    Fetch point occurrences for a location (with Geohash precision 8 or
    variable Geohash precision) as GeoJSON.

    :param x: [float] latitudes of a location
    :param y: [float] longitude of a location
    :param z: [float] vertical datum (geodatic datum WGS84)
    :param scientificname: [string] Scientific name. Leave empty to include all
        taxa.
    :param taxonid: [string] Taxon AphiaID.
    :param datasetid: [string] Dataset UUID.
    :param nodeid: [string] Node UUID.
    :param startdate: [string] Start date formatted as YYYY-MM-DD.
    :param enddate: [string] End date formatted as YYYY-MM-DD.
    :param startdepth: [integer] Start depth, in meters. Depth below sea level are treated
        as positive numbers.
    :param enddepth: [integer] End depth, in meters. Depth below sea level are treated
        as positive numbers.
    :param geometry: [string] Geometry, formatted as WKT or GeoHash.
    :param redlist: [boolean] Red List species only, True/False.
    :param hab: [boolean] HAB species only, true/false.
    :param wrims: [boolean] WRiMS species only, True/False.
    :param event: [string] Include pure event records (include) or get pure
        event records exclusively (true).
    :param flags: [string] Comma separated list of quality flags which need
        to be set.
    :param exclude: [string] Comma separated list of quality flags to be
        excluded.

    :return: A dictionary

    Usage::

        from pyobis import occurrences
        occurrences.point(x=1.77,y=54.22,scientificname = 'Mola mola')

    """
    z = str(z) if z else ""
    url = obis_baseurl + f"occurrence/point/{str(x)}/{str(y)}/{z}"
    scientificname = handle_arrstr(scientificname)
    taxonid = handle_arrint(taxonid)
    args = {
        "scientificname": scientificname,
        "taxonid": taxonid,
        "datasetid": datasetid,
        "nodeid": nodeid,
        "startdate": startdate,
        "enddate": enddate,
        "startdepth": startdepth,
        "enddepth": enddepth,
        "geometry": geometry,
        "redlist": redlist,
        "hab": hab,
        "wrims": wrims,
        "event": event,
        "flags": flags,
        "exclude": exclude,
    }

    return OccResponse(url, args, isSearch=False, hasMapper=False, isKML=False)


def tile(
    x,
    y,
    z,
    mvt=0,
    scientificname=None,
    taxonid=None,
    datasetid=None,
    nodeid=None,
    startdate=None,
    enddate=None,
    startdepth=None,
    enddepth=None,
    geometry=None,
    redlist=None,
    hab=None,
    wrims=None,
    event=None,
    flags=None,
    exclude=None,
    **kwargs,
):
    """
    Fetch point occurrences for a tile (aggregated using variable Geohash
    precision based on zoom level) as GeoJSON or MVT.

    :param x: [float] latitudes of a location
    :param y: [float] longitude of a location
    :param z: [float] vertical datum (geodatic datum WGS84)
    :param scientificname: [string] Scientific name. Leave empty to include
        all taxa.
    :param taxonid: [string] Taxon AphiaID.
    :param datasetid: [string] Dataset UUID.
    :param nodeid: [string] Node UUID.
    :param startdate: [string] Start date formatted as YYYY-MM-DD.
    :param enddate: [string] End date formatted as YYYY-MM-DD.
    :param startdepth: [integer] Start depth, in meters. Depth below sea level are treated
        as positive numbers.
    :param enddepth: [integer] End depth, in meters. Depth below sea level are treated
        as positive numbers.
    :param geometry: [string] Geometry, formatted as WKT or GeoHash.
    :param redlist: [boolean] Red List species only, True/False.
    :param hab: [boolean] HAB species only, true/false.
    :param wrims: [boolean] WRiMS species only, True/False.
    :param event: [string] Include pure event records (include) or get pure
        event records exclusively (true).
    :param flags: [string] Comma separated list of quality flags which need to
        be set.
    :param exclude: [string] Comma separated list of quality flags to be
        excluded.

    :return: A dictionary

    Usage::

        from pyobis import occurrences
        occurrences.tile(x=1.77,y=52.26,z=0.5,mvt=0, scientificname = 'Mola mola')
        occurrences.tile(x=1.77,y=52.26,z=0.5,mvt=1, scientificname = 'Mola mola')
    """
    url = obis_baseurl + f"occurrence/tile/{str(x)}/{str(y)}/{str(z)}"
    scientificname = handle_arrstr(scientificname)
    taxonid = handle_arrint(taxonid)
    args = {
        "scientificname": scientificname,
        "taxonid": taxonid,
        "datasetid": datasetid,
        "nodeid": nodeid,
        "startdate": startdate,
        "enddate": enddate,
        "startdepth": startdepth,
        "enddepth": enddepth,
        "geometry": geometry,
        "redlist": redlist,
        "hab": hab,
        "wrims": wrims,
        "event": event,
        "flags": flags,
        "exclude": exclude,
    }
    if mvt:
        url += ".mvt"
        return OccResponse(url, args, isSearch=False, hasMapper=False, isKML=True)

    return OccResponse(url, args, isSearch=False, hasMapper=False, isKML=False)


def centroid(
    scientificname=None,
    taxonid=None,
    datasetid=None,
    nodeid=None,
    startdate=None,
    enddate=None,
    startdepth=None,
    enddepth=None,
    geometry=None,
    redlist=None,
    hab=None,
    wrims=None,
    event=None,
    flags=None,
    exclude=None,
    **kwargs,
):
    """
    Determine the centroid for a selection of occurrence records.

    :param scientificname: [string] Scientific name. Leave empty to include all
        taxa.
    :param taxonid: [string] Taxon AphiaID.
    :param datasetid: [string] Dataset UUID.
    :param nodeid: [string] Node UUID.
    :param startdate: [string] Start date formatted as YYYY-MM-DD.
    :param enddate: [string] End date formatted as YYYY-MM-DD.
    :param startdepth: [integer] Start depth, in meters. Depth below sea level are treated
        as positive numbers.
    :param enddepth: [integer] End depth, in meters. Depth below sea level are treated
        as positive numbers.
    :param geometry: [string] Geometry, formatted as WKT or GeoHash.
    :param redlist: [boolean] Red List species only, True/False.
    :param hab: [boolean] HAB species only, true/false.
    :param wrims: [boolean] WRiMS species only, True/False.
    :param event: [string] Include pure event records (include) or get pure
        event records exclusively (true).
    :param flags: [string] Comma separated list of required quality flags.
    :param exclude: [string] Comma separated list of quality flags to be
        excluded.

    :return: A dictionary

    Usage::

        from pyobis import occurrences
        occurrences.centroid(scientificname = 'Mola mola')
    """
    url = obis_baseurl + "occurrence/centroid"
    scientificname = handle_arrstr(scientificname)
    taxonid = handle_arrint(taxonid)
    args = {
        "scientificname": scientificname,
        "taxonid": taxonid,
        "datasetid": datasetid,
        "nodeid": nodeid,
        "startdate": startdate,
        "enddate": enddate,
        "startdepth": startdepth,
        "enddepth": enddepth,
        "geometry": geometry,
        "redlist": redlist,
        "hab": hab,
        "wrims": wrims,
        "event": event,
        "flags": flags,
        "exclude": exclude,
    }

    return OccResponse(url, args, isSearch=False, hasMapper=False, isKML=False)


def lookup_taxon(scientificname):
    """
    Lookup for taxon metadata with scientificname

    :param scientificname: [String] Scientific Name

    :return: A dictionary of taxon metadata for the best matches to the input

    Usage::

        from pyobis import occurrences
        lookup_data = occurrences.lookup_taxon(scientificname="Mola mola")
        print(lookup_data)
    """
    res = requests.get(f"https://api.obis.org/v3/taxon/complete/{scientificname}")
    return res.json()



"""Tests for checklist module"""
import requests

from pyobis import checklist


def test_checklist():
    """
    checklist.list - basic test for data, check type, size and other methods
    """
    query = checklist.list(scientificname="Mola mola")
    assert not query.data
    query.execute()
    assert "dict" == query.data.__class__.__name__
    assert 2 == len(query.data)
    assert dict == query.data["results"][0].__class__
    assert int == query.data["results"][0]["taxonID"].__class__
    assert "Mola mola" == query.data["results"][0]["species"]
    assert requests.get(query.api_url).status_code == 200


def test_checklist_redlist():
    """
    checklist.redlist - basic test for data, check type, size and other methods
    """
    query = checklist.redlist(scientificname="Mola mola")
    assert not query.data
    query.execute()
    assert "dict" == query.data.__class__.__name__
    assert 2 == len(query.data)
    assert dict == query.data["results"][0].__class__
    assert int == query.data["results"][0]["taxonID"].__class__
    assert "Mola mola" == query.data["results"][0]["species"]
    assert requests.get(query.api_url).status_code == 200


def test_checklist_newest():
    """
    checklist.newest - basic test for data, check type, size and other methods
    """
    query = checklist.newest(scientificname="Mola mola")
    assert not query.data
    query.execute()
    assert "dict" == query.data.__class__.__name__
    assert 1 == len(query.data)
    assert dict == query.data["results"][0].__class__
    assert int == query.data["results"][0]["taxonID"].__class__
    assert "Mola mola" == query.data["results"][0]["species"]
    assert requests.get(query.api_url).status_code == 200
    assert query.to_pandas().__class__.__name__ == "DataFrame"



"""Tests for dataset module"""
import requests

from pyobis import dataset


def test_dataset_get_data():
    "dataset.get - test data"
    query = dataset.get(id="ec9df3b9-3b2b-4d83-881b-27bcbcd57b95")
    assert not query.data
    query.execute()
    assert query.data.__class__.__name__ == "dict"
    assert 2 == len(query.data)
    assert dict == query.data["results"][0].__class__
    assert query.to_pandas().__class__.__name__ == "DataFrame"


def test_dataset_get_url():
    query = dataset.get(id="ec9df3b9-3b2b-4d83-881b-27bcbcd57b95")
    assert requests.get(query.api_url).status_code == 200
    assert requests.get(query.mapper_url).status_code == 200


def test_dataset_search_data():
    "dataset.search - test for data"
    query = dataset.search(scientificname="Mola mola")
    assert not query.data
    query.execute()
    assert "dict" == query.data.__class__.__name__
    assert 2 == len(query.data)
    assert dict == query.data["results"][0].__class__
    assert query.to_pandas().__class__.__name__ == "DataFrame"


def test_dataset_search_url():
    query = dataset.search(scientificname="Mola mola")
    assert requests.get(query.api_url).status_code == 200
    assert not query.mapper_url



from setuptools import setup

setup(
    # The package metadata is specified in setup.cfg but GitHub's downstream dependency graph
    # does not work unless we put the name this here too.
    name="pyobis",
    use_scm_version={
        "write_to": "pyobis/_version.py",
        "write_to_template": '__version__ = "{version}"',
        "tag_regex": r"^(?P<prefix>v)?(?P<version>[^\+]+)(?P<suffix>.*)?$",
    },
)
